#include <stdio.h>

#ifndef __TripleData_h__ 
#define __TripleData_h__ 

//#include "platform.h"

#include "WHO.h"

namespace HardenedData
{
	// enum WHO;
	template <class DataType> class TripleData;
}

namespace HardenedData
{
	template <class DataType> class TripleData
	{
		private: DataType d1; 

		private: DataType d2; 

		private: DataType d3; 


		/// <summary>
		/// Casts a redundant TripleData<DataType> variable a to a single DataType by voting.
		///         TripleData<DataType> a;
		///         DataType b;
		///         b = (DataType) a;
		/// If at most one replica of a is corrupted, voting will recover a correct value. If all three replicas are different from each other, the first (potentially corrupted) replica is returned.
		/// The class' internal storage is NOT affected so, if the variable is corrupted, the casting operator will return a correct value, while the class is left corrupted.
		/// To recover internal class' storage to a correct situation (namely, with all three replicas correct), one shall explicitly use the sync() : DataType operator.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		operator DataType() const {
			if (d1==d2) return d1;
			else if (d1==d3) return d1;
			else return d2;
		}


		/// <summary>
		/// Overwrites the assignation operator a = b where both a and b are redundant TripleData<DataType>.
		/// It copies a value into the class' internal storage, by triplicating the right-hand value into the class' internal replicas.
		/// When using this operator, the right-hand variable is not verified to be correct; each of the three replicas of the right-hand value is copied as it is into the corresponding replica of the left-hand variable, for a faster execution. As a consequence, assigning a corrupted right-hand value to the left-hand variable, leaves the value corrupted.
		/// To assign a correct left-hand value from a corrupted right-hand location, one shall use the following construct:
		///         TripleData<DataType> a, b;
		///         a = (DataType) b;
		/// where type casting TripleData<DataType> b to (DataType) recovers a correct value from the three replicas (by voting). The correct value is then copied identically to the three replicas of a.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType>& operator=(const HardenedData::TripleData<DataType>& val) {
			d1 = val.d1;
			d2 = val.d2;
			d3 = val.d3;
			return *this;  // Return a reference to myself
		}


		/// <summary>
		/// Overwrites the assignation operator a = b where a is a redundant TripleData<DataType> while b is a non-redundant DataType.
		/// It copies a variable or a constant into the class' internal storage, by triplicating the right-hand value into the class' internal replicas.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType>& operator=(const DataType val) {
			d1 = val;
			d2 = val;
			d3 = val;
			return *this;  // Return a reference to myself
		}


		/// <summary>
		/// Overwrites the algebraic operator a + b when a is redundant TripleData<DataType> while b is DataType.
		/// It sums up each replica of the left-hand variable a to the right-hand variable (or constant) b and returns the result as a redundant TripleData<DataType>. Neither a nor b are affected.
		/// When using this operator, the left-hand variable is not verified to be correct; each of the three replicas of the left-hand value is summed up as it is to the right-hand variable, for a faster execution. As a consequence, summing up a corrupted left-hand value to a right-hand variable, leaves the value corrupted.
		/// To sum up and correct a corrupted left-hand value to a right-hand variable (or constant), one shall use the following construct:
		///         TripleData<DataType> a, b, c;
		///         DataType b;
		///         c = (DataType) a + b; // or, alternatively
		///         c = (DataType) a + 2;
		/// where type casting TripleData<DataType> a to (DataType) recovers a correct value from its three replicas (by voting). The correct value is then copied identically to the three replicas of c.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType> operator+(const DataType& val) const {
			TripleData<DataType> result;
			result.d1 = d1 + val;
			result.d2 = d2 + val;
			result.d3 = d3 + val;
			return result;
		}


		/// <summary>
		/// Overwrites the algebraic operator a + b when both a and b are redundant TripleData<DataType>.
		/// It sums up each replica of the left-hand variable a to the corresponding replica of the right-hand variable b and returns the result as a redundant TripleData<DataType>. Neither a nor b are affected.
		/// When using this operator, neither the left-hand nor the right-hand variables are verified to be correct; each of the three replicas of the left-hand value is summed up as it is to the corresponding replica of the right-hand variable, for a faster execution. As a consequence, summing up a corrupted left-hand value to a corrupted right-hand variable, leaves the value corrupted.
		/// Note that corruption propagation with sums is more risky than with assignation operator = for the following reason: imagine that a has the first replica corrupted, while the second and third replica are correct, while b only has its second replica corrupted. Both a and b can be individually recovered by voting, but summing up the first, second and third replicas of a and b together, will return a corrupted first replica (because of a's corruption) together with a corrupted second replica (because of b's corruption), so the result will NOT be recoverable!
		/// To sum up and correct a corrupted left-hand value to a corrupted right-hand variable, one shall use the following construct:
		///         TripleData<DataType> a, b, c;
		///         c = (DataType) a + (DataType) b;
		/// where type casting TripleData<DataType> a and b to (DataType) recovers a correct value from the three replicas of both variables (by voting). The correct value is then copied identically to the three replicas of c.
		/// Alternatively, for a faster execution, the following construct:
		///         TripleData<DataType> a, b, c;
		///         c = (DataType) a + b;
		/// will only recover a but not b; the result might be potentially corrupted but recoverable, as at most one replica of the result will be corrupted.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType> operator+(const HardenedData::TripleData<DataType>& val) const {
			TripleData<DataType> result;
			result.d1 = d1 + val.d1;
			result.d2 = d2 + val.d2;
			result.d3 = d3 + val.d3;
			return result;
			
			// an alternative, apparently more efficient and more rubust version:
			//result = (DataType) *this + (DataType) val;
			//return result;
		}


		/// <summary>
		/// Overwrites the pre-increment operator ++a, when a is redundant TripleData<DataType>.
		/// It individually increments each replica of the left-hand variable a and returns the value before incrementing as a redundant TripleData<DataType>. The left-hand operator is therefore modified.
		/// When using this operator, the left-hand variable is not verified to be correct; each of the three replicas of the left-hand value are incremented separately, for a faster execution. As a consequence, incrementing a corrupted left-hand value, leaves the value corrupted.
		/// To have a correct value after incrementing, one shall explicitly use the sync() : DataType operator:
		///         TripleData<DataType> a;
		///         ++a;
		///         a.sync();
		/// where sync() : DataType restores a correct value into a.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType> operator++() {
			TripleData<DataType> temp;
			temp = (*this);
			d1++;
			d2++;
			d3++;
			return temp;
		}


		/// <summary>
		/// Overwrites the post-increment operator a++, when a is redundant TripleData<DataType>.
		/// It individually increments each replica of the left-hand variable a and returns the incremented value as a redundant TripleData<DataType>. The left-hand operator is therefore modified.
		/// When using this operator, the left-hand variable is not verified to be correct; each of the three replicas of the left-hand value are incremented separately, for a faster execution. As a consequence, incrementing a corrupted left-hand value, leaves the value corrupted.
		/// To have a correct value after incrementing, one shall explicitly use the sync() : DataType operator:
		///         TripleData<DataType> a;
		///         a++;
		///         a.sync();
		/// where sync() : DataType restores a correct value into a.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType>& operator++(int dummy) {
			d1++;
			d2++;
			d3++;
			return *this;  // Return a reference to myself
		}


		/// <summary>
		/// Overwrites the algebraic operator a += b when a is redundant TripleData<DataType> while b is DataType.
		/// It sums up each replica of the left-hand variable a to the right-hand variable (or constant) b, stores the result back to a and returns the result as a redundant TripleData<DataType>. Variable a is affected, while b is not affected.
		/// When using this operator, the left-hand variables is not verified to be correct; each of the three replicas of the left-hand value is summed up as it is to the right-hand variable, for a faster execution. As a consequence, summing up a corrupted left-hand value to a right-hand variable, leaves the value corrupted.
		/// To sum up and correct a corrupted left-hand value to a right-hand variable, one shall use either of the following constructs:
		///         TripleData<DataType> a;
		///         DataType b;
		///         a += b;
		///         a.sync();
		/// or
		///         TripleData<DataType> a;
		///         DataType b;
		///         a = (DataType) a + b;
		/// which recovers a before using its value.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType>& operator+=(const DataType& val) {
			d1 += val;
			d2 += val;
			d3 += val;
			return *this;  // Return a reference to myself
		}


		/// <summary>
		/// Overwrites the algebraic operator a += b when both a and b are redundant TripleData<DataType>.
		/// It sums up each replica of the left-hand variable a to the corresponding replica of the right-hand variable b, stores the result back to a and returns the result as a redundant TripleData<DataType>. Variable a is affected, while b is not affected.
		/// When using this operator, neither the left-hand nor the right-hand variables are verified to be correct; each of the three replicas of the left-hand value is summed up as it is to the corresponding replica of the right-hand variable, for a faster execution. As a consequence, summing up a corrupted left-hand value to a corrupted right-hand variable, leaves the value corrupted.
		/// Note that corruption propagation with sums is more risky than with assignation operator = for the following reason: imagine that a has the first replica corrupted, while the second and third replica are correct, while b only has its second replica corrupted. Both a and b can be individually recovered by voting, but summing up the first, second and third replicas of a and b together, will return a corrupted first replica (because of a's corruption) together with a corrupted second replica (because of b's corruption), so the result will NOT be recoverable!
		/// To sum up and correct a corrupted left-hand value to a corrupted right-hand variable, one shall use the following construct:
		///         TripleData<DataType> a, b;
		///         a += (DataType) b;
		/// where type casting TripleData<DataType> b to (DataType) recovers a correct value from the three replicas of b (by voting). The correct value is then summed identically to the three replicas of a.
		/// If b is corrupted, its correct value is recovered, while if a is corrupted, it will remain corrupted. To have a fully correct value stored into a, one shall alternatively use either of the following constructs:
		///         TripleData<DataType> a, b;
		///         a += (DataType) b;
		///         a.sync();
		/// or
		///         TripleData<DataType> a, b;
		///         a = (DataType) a + (DataType) b;
		/// which recover both a and b.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType>& operator+=(const HardenedData::TripleData<DataType>& val) {
			d1 += val.d1;
			d2 += val.d2;
			d3 += val.d3;
			return *this;  // Return a reference to myself
		}


		/// <summary>
		/// Overwrites the algebraic operator a - b when a is redundant TripleData<DataType> while b is DataType.
		/// It subtracts each replica of the left-hand variable a from the right-hand variable (or constant) b and returns the result as a redundant TripleData<DataType>. Neither a nor b are affected.
		/// When using this operator, the left-hand variable is not verified to be correct; each of the three replicas of the left-hand value is subtracted as it is from the right-hand variable, for a faster execution. As a consequence, subtracting a corrupted left-hand value from a right-hand variable, leaves the value corrupted.
		/// To subtract and correct a corrupted left-hand value from a right-hand variable (or constant), one shall use the following construct:
		///         TripleData<DataType> a, b, c;
		///         DataType b;
		///         c = (DataType) a - b; // or, alternatively
		///         c = (DataType) a - 2;
		/// where type casting TripleData<DataType> a to (DataType) recovers a correct value from its three replicas (by voting). The correct value is then copied identically to the three replicas of c.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType> operator-(const DataType& val) const {
			TripleData<DataType> result;
			result.d1 = d1 - val;
			result.d2 = d2 - val;
			result.d3 = d3 - val;
			return result;
		}


		/// <summary>
		/// Overwrites the algebraic operator a - b when both a and b are redundant TripleData<DataType>.
		/// It subtracts each replica of the left-hand variable a from the corresponding replica of the right-hand variable (or constant) b and returns the result as a redundant TripleData<DataType>. Neither a nor b are affected.
		/// When using this operator, neither the left-hand nor the right-hand variables are verified to be correct; each of the three replicas of the left-hand value is subtracted as it is from the corresponding replica of the right-hand variable, for a faster execution. As a consequence, subtracting a corrupted left-hand value from a corrupted right-hand variable, leaves the value corrupted.
		/// Note that corruption propagation with subtraction is more risky than with assignation operator = for the following reason: imagine that a has the first replica corrupted, while the second and third replica are correct, while b only has its second replica corrupted. Both a and b can be individually recovered by voting, but subtracting the first, second and third replicas of a from b, will return a corrupted first replica (because of a's corruption) together with a corrupted second replica (because of b's corruption), so the result will NOT be recoverable!
		/// To subtract and correct a corrupted left-hand value from a corrupted right-hand variable, one shall use the following construct:
		///         TripleData<DataType> a, b, c;
		///         c = (DataType) a - (DataType) b;
		/// where type casting TripleData<DataType> a and b to (DataType) recovers a correct value from the three replicas of both variables (by voting). The correct value is then copied identically to the three replicas of c.
		/// Alternatively, for a faster execution, the following construct:
		///         TripleData<DataType> a, b, c;
		///         c = (DataType) a - b;
		/// will only recover a but not b; the result might be potentially corrupted but recoverable, as at most one replica of the result will be corrupted.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType> operator-(const HardenedData::TripleData<DataType>& val) const {
			TripleData<DataType> result;
			result.d1 = d1 - val.d1;
			result.d2 = d2 - val.d2;
			result.d3 = d3 - val.d3;
			return result;
		}


		/// <summary>
		/// Overwrites the pre-decrement operator --a, when a is redundant TripleData<DataType>.
		/// It individually decrements each replica of the left-hand variable a and returns the value before decrementing as a redundant TripleData<DataType>. The left-hand operator is therefore modified.
		/// When using this operator, the left-hand variable is not verified to be correct; each of the three replicas of the left-hand value are decremented separately, for a faster execution. As a consequence, decrementing a corrupted left-hand value, leaves the value corrupted.
		/// To have a correct value after decrementing, one shall explicitly use the sync() : DataType operator:
		///         TripleData<DataType> a;
		///         --a;
		///         a.sync();
		/// where sync() : DataType restores a correct value into a.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType>& operator--() {
			d1--;
			d2--;
			d3--;
			return *this;  // Return a reference to myself
		}


		/// <summary>
		/// Overwrites the post-decrement operator a--, when a is redundant TripleData<DataType>.
		/// It individually decrements each replica of the left-hand variable a and returns the decremented value as a redundant TripleData<DataType>. The left-hand operator is therefore modified.
		/// When using this operator, the left-hand variable is not verified to be correct; each of the three replicas of the left-hand value are decremented separately, for a faster execution. As a consequence, decrementing a corrupted left-hand value, leaves the value corrupted.
		/// To have a correct value after decrementing, one shall explicitly use the sync() : DataType operator:
		///         TripleData<DataType> a;
		///         a--;
		///         a.sync();
		/// where sync() : DataType restores a correct value into a.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType>& operator--(int dummy) {
			d1--;
			d2--;
			d3--;
			return *this;  // Return a reference to myself
		}


		/// <summary>
		/// Overwrites the algebraic operator a -= b when a is redundant TripleData<DataType> while b is DataType.
		/// It subtracts each replica of the left-hand variable a from the right-hand variable (or constant) b, stores the result back to a and returns the result as a redundant TripleData<DataType>. Variable a is affected, while b is not affected.
		/// When using this operator, the left-hand variables is not verified to be correct; each of the three replicas of the left-hand value is subtracted as it is from the right-hand variable, for a faster execution. As a consequence, subtracting a corrupted left-hand value from a right-hand variable, leaves the value corrupted.
		/// To subtract and correct a corrupted left-hand value from a right-hand variable, one shall use either of the following constructs:
		///         TripleData<DataType> a;
		///         DataType b;
		///         a -= b;
		///         a.sync();
		/// or
		///         TripleData<DataType> a;
		///         DataType b;
		///         a = (DataType) a - b;
		/// which recover a before using its value.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType>& operator-=(const DataType& val) {
			d1 -= val;
			d2 -= val;
			d3 -= val;
			return *this;  // Return a reference to myself
		}


		/// <summary>
		/// Overwrites the algebraic operator a -= b when both a and b are redundant TripleData<DataType>.
		/// It subtracts each replica of the left-hand variable a from the corresponding replica of the right-hand variable (or constant) b, stores the result back to a and returns the result as a redundant TripleData<DataType>. Variable a is affected, while b is not affected.
		/// When using this operator, neither the left-hand nor the right-hand variables are verified to be correct; each of the three replicas of the left-hand value is subtracted as it is from the corresponding replica of the right-hand variable, for a faster execution. As a consequence, subtracting a corrupted left-hand value from a corrupted right-hand variable, leaves the value corrupted.
		/// Note that corruption propagation with subtractions is more risky than with assignation operator = for the following reason: imagine that a has the first replica corrupted, while the second and third replica are correct, while b only has its second replica corrupted. Both a and b can be individually recovered by voting, but subtracting the first, second and third replicas of b from a, will return a corrupted first replica (because of a's corruption) together with a corrupted second replica (because of b's corruption), so the result will NOT be recoverable!
		/// To subtract and correct a corrupted left-hand value from a corrupted right-hand variable, one shall use the following construct:
		///         TripleData<DataType> a, b;
		///         a -= (DataType) b;
		/// where type casting TripleData<DataType> b to (DataType) recovers a correct value from the three replicas of b (by voting). The correct value is then subtracted identically from the three replicas of a.
		/// If b is corrupted, its correct value is recovered, while if a is corrupted, it will remain corrupted. To have a fully correct value stored into a, one shall alternatively use either of the following constructs:
		///         TripleData<DataType> a, b;
		///         a -= (DataType) b;
		///         a.sync();
		/// or
		///         TripleData<DataType> a, b;
		///         a = (DataType) a - (DataType) b;
		/// which recover both a and b.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType>& operator-=(const HardenedData::TripleData<DataType>& val) {
			d1 -= val.d1;
			d2 -= val.d2;
			d3 -= val.d3;
			return *this;  // Return a reference to myself
		}


		/// <summary>
		/// Overwrites the algebraic operator a * b when a is redundant TripleData<DataType> while b is DataType.
		/// It multiplies each replica of the left-hand variable a by the right-hand variable (or constant) b and returns the result as a redundant TripleData<DataType>. Neither a nor b are affected.
		/// When using this operator, the left-hand variable is not verified to be correct; each of the three replicas of the left-hand value is multiplied as it is by the right-hand variable, for a faster execution. As a consequence, multiplying a corrupted left-hand value by a right-hand variable, leaves the value corrupted.
		/// To multiply and correct a corrupted left-hand value by a right-hand variable (or constant), one shall use the following construct:
		///         TripleData<DataType> a, b, c;
		///         DataType b;
		///         c = (DataType) a * b; // or, alternatively
		///         c = (DataType) a * 2;
		/// where type casting TripleData<DataType> a to (DataType) recovers a correct value from its three replicas (by voting). The correct value is then copied identically to the three replicas of c.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType> operator*(const DataType& val) const {
			TripleData<DataType> result;
			result.d1 = d1 * val;
			result.d2 = d2 * val;
			result.d3 = d3 * val;
			return result;
		}


		/// <summary>
		/// Overwrites the algebraic operator a * b when both a and b are redundant TripleData<DataType>.
		/// It multiplies each replica of the left-hand variable a by the corresponding replica of the right-hand variable b and returns the result as a redundant TripleData<DataType>. Neither a nor b are affected.
		/// When using this operator, neither the left-hand nor the right-hand variables are verified to be correct; each of the three replicas of the left-hand value is multiplied as it is by the corresponding replica of the right-hand variable, for a faster execution. As a consequence, multiplying a corrupted left-hand value by a corrupted right-hand variable, leaves the value corrupted.
		/// Note that corruption propagation with multiplications is more risky than with assignation operator = for the following reason: imagine that a has the first replica corrupted, while the second and third replica are correct, while b only has its second replica corrupted. Both a and b can be individually recovered by voting, but multiplying the first, second and third replicas of a and b together, will return a corrupted first replica (because of a's corruption) together with a corrupted second replica (because of b's corruption), so the result will NOT be recoverable!
		/// To multiply and correct a corrupted left-hand value by a corrupted right-hand variable, one shall use the following construct:
		///         TripleData<DataType> a, b, c;
		///         c = (DataType) a * (DataType) b;
		/// where type casting TripleData<DataType> a and b to (DataType) recovers a correct value from the three replicas of both variables (by voting). The correct value is then copied identically to the three replicas of c.
		/// Alternatively, for a faster execution, the following construct:
		///         TripleData<DataType> a, b, c;
		///         c = (DataType) a * b;
		/// will only recover a but not b; the result might be potentially corrupted but recoverable, as at most one replica of the result will be corrupted.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType> operator*(const HardenedData::TripleData<DataType>& val) const {
			TripleData<DataType> result;
			result.d1 = d1 * val.d1;
			result.d2 = d2 * val.d2;
			result.d3 = d3 * val.d3;
			return result;
		}


		/// <summary>
		/// Overwrites the algebraic operator a *= b when a is redundant TripleData<DataType> while b is DataType.
		/// It multiplies each replica of the left-hand variable a by the right-hand variable (or constant) b, stores the result back to a and returns the result as a redundant TripleData<DataType>. Variable a is affected, while b is not affected.
		/// When using this operator, the left-hand variables is not verified to be correct; each of the three replicas of the left-hand value is multiplied as it is by the right-hand variable, for a faster execution. As a consequence, multiplying up a corrupted left-hand value by a right-hand variable, leaves the value corrupted.
		/// To multiply and correct a corrupted left-hand value by a right-hand variable, one shall use either of the following constructs:
		///         TripleData<DataType> a;
		///         DataType b;
		///         a *= b;
		///         a.sync();
		/// or
		///         TripleData<DataType> a;
		///         DataType b;
		///         a = (DataType) a * b;
		/// which recovers a before using its value.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType>& operator*=(const DataType& val) {
			d1 *= val;
			d2 *= val;
			d3 *= val;
			return *this;  // Return a reference to myself
		}


		/// <summary>
		/// Overwrites the algebraic operator a *= b when both a and b are redundant TripleData<DataType>.
		/// It multiplies each replica of the left-hand variable a by the corresponding replica of the right-hand variable (or constant) b, stores the result back to a and returns the result as a redundant TripleData<DataType>. Variable a is affected, while b is not affected.
		/// When using this operator, neither the left-hand nor the right-hand variables are verified to be correct; each of the three replicas of the left-hand value is multiplied as it is from the corresponding replica of the right-hand variable, for a faster execution. As a consequence, multiplying a corrupted left-hand value by a corrupted right-hand variable, leaves the value corrupted.
		/// Note that corruption propagation with multiplications is more risky than with assignation operator = for the following reason: imagine that a has the first replica corrupted, while the second and third replica are correct, while b only has its second replica corrupted. Both a and b can be individually recovered by voting, but multiplying the first, second and third replicas of a and b together, will return a corrupted first replica (because of a's corruption) together with a corrupted second replica (because of b's corruption), so the result will NOT be recoverable!
		/// To multiply and correct a corrupted left-hand value by a corrupted right-hand variable, one shall use the following construct:
		///         TripleData<DataType> a, b;
		///         a *= (DataType) b;
		/// where type casting TripleData<DataType> b to (DataType) recovers a correct value from the three replicas of b (by voting). The correct value is then multiplied identically by the three replicas of a.
		/// If b is corrupted, its correct value is recovered, while if a is corrupted, it will remain corrupted. To have a fully correct value stored into a, one shall alternatively use either of the following constructs:
		///         TripleData<DataType> a, b;
		///         a *= (DataType) b;
		///         a.sync();
		/// or
		///         TripleData<DataType> a, b;
		///         a = (DataType) a * (DataType) b;
		/// which recover both a and b.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType>& operator*=(const HardenedData::TripleData<DataType>& val) {
			d1 *= val.d1;
			d2 *= val.d2;
			d3 *= val.d3;
			return *this;  // Return a reference to myself
		}


		/// <summary>
		/// Overwrites the algebraic operator a / b when a is redundant TripleData<DataType> while b is DataType.
		/// It divides each replica of the left-hand variable a by the right-hand variable (or constant) b and returns the result as a redundant TripleData<DataType>. Neither a nor b are affected.
		/// When using this operator, the left-hand variable is not verified to be correct; each of the three replicas of the left-hand value is divided as it is by the right-hand variable, for a faster execution. As a consequence, dividing a corrupted left-hand value by a right-hand variable, leaves the value corrupted.
		/// To divide and correct a corrupted left-hand value by a right-hand variable (or constant), one shall use the following construct:
		///         TripleData<DataType> a, b, c;
		///         DataType b;
		///         c = (DataType) a / b; // or, alternatively
		///         c = (DataType) a / 2;
		/// where type casting TripleData<DataType> a to (DataType) recovers a correct value from its three replicas (by voting). The correct value is then copied identically to the three replicas of c.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType> operator/(const DataType& val) const {
			TripleData<DataType> result;
			result.d1 = d1 / val;
			result.d2 = d2 / val;
			result.d3 = d3 / val;
			return result;
		}


		/// <summary>
		/// Overwrites the algebraic operator a / b when both a and b are redundant TripleData<DataType>.
		/// It divides each replica of the left-hand variable a by the corresponding replica of the right-hand variable b and returns the result as a redundant TripleData<DataType>. Neither a nor b are affected.
		/// When using this operator, neither the left-hand nor the right-hand variables are verified to be correct; each of the three replicas of the left-hand value is divided as it is by the corresponding replica of the right-hand variable, for a faster execution. As a consequence, dividing a corrupted left-hand value by a corrupted right-hand variable, leaves the value corrupted.
		/// Note that corruption propagation with divisions is more risky than with assignation operator = for the following reason: imagine that a has the first replica corrupted, while the second and third replica are correct, while b only has its second replica corrupted. Both a and b can be individually recovered by voting, but dividing the first, second and third replicas of a and b together, will return a corrupted first replica (because of a's corruption) together with a corrupted second replica (because of b's corruption), so the result will NOT be recoverable!
		/// To divide and correct a corrupted left-hand value by a corrupted right-hand variable, one shall use the following construct:
		///         TripleData<DataType> a, b, c;
		///         c = (DataType) a / (DataType) b;
		/// where type casting TripleData<DataType> a and b to (DataType) recovers a correct value from the three replicas of both variables (by voting). The correct value is then copied identically to the three replicas of c.
		/// Alternatively, for a faster execution, the following construct:
		///         TripleData<DataType> a, b, c;
		///         c = (DataType) a / b;
		/// will only recover a but not b; the result might be potentially corrupted but recoverable, as at most one replica of the result will be corrupted.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType> operator/(const HardenedData::TripleData<DataType>& val) const {
			TripleData<DataType> result;
			result.d1 = d1 / val.d1;
			result.d2 = d2 / val.d2;
			result.d3 = d3 / val.d3;
			return result;
		}


		/// <summary>
		/// Overwrites the algebraic operator a /= b when a is redundant TripleData<DataType> while b is DataType.
		/// It divides each replica of the left-hand variable a by the right-hand variable (or constant) b, stores the result back to a and returns the result as a redundant TripleData<DataType>. Variable a is affected, while b is not affected.
		/// When using this operator, the left-hand variables is not verified to be correct; each of the three replicas of the left-hand value is divided as it is by the right-hand variable, for a faster execution. As a consequence, dividing a corrupted left-hand value by a right-hand variable, leaves the value corrupted.
		/// To divide and correct a corrupted left-hand value by a right-hand variable, one shall use either of the following constructs:
		///         TripleData<DataType> a;
		///         DataType b;
		///         a /= b;
		///         a.sync();
		/// or
		///         TripleData<DataType> a;
		///         DataType b;
		///         a = (DataType) a / b;
		/// which recovers a before using its value.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType>& operator/=(const DataType& val) {
			d1 /= val;
			d2 /= val;
			d3 /= val;
			return *this;  // Return a reference to myself
		}


		/// <summary>
		/// Overwrites the algebraic operator a /= b when both a and b are redundant TripleData<DataType>.
		/// It divides each replica of the left-hand variable a by the corresponding replica of the right-hand variable (or constant) b, stores the result back to a and returns the result as a redundant TripleData<DataType>. Variable a is affected, while b is not affected.
		/// When using this operator, neither the left-hand nor the right-hand variables are verified to be correct; each of the three replicas of the left-hand value is divided as it is from the corresponding replica of the right-hand variable, for a faster execution. As a consequence, dividing a corrupted left-hand value by a corrupted right-hand variable, leaves the value corrupted.
		/// Note that corruption propagation with divisions is more risky than with assignation operator = for the following reason: imagine that a has the first replica corrupted, while the second and third replica are correct, while b only has its second replica corrupted. Both a and b can be individually recovered by voting, but dividing the first, second and third replicas of a and b together, will return a corrupted first replica (because of a's corruption) together with a corrupted second replica (because of b's corruption), so the result will NOT be recoverable!
		/// To divide and correct a corrupted left-hand value by a corrupted right-hand variable, one shall use the following construct:
		///         TripleData<DataType> a, b;
		///         a /= (DataType) b;
		/// where type casting TripleData<DataType> b to (DataType) recovers a correct value from the three replicas of b (by voting). The correct value is then divided identically by the three replicas of a.
		/// If b is corrupted, its correct value is recovered, while if a is corrupted, it will remain corrupted. To have a fully correct value stored into a, one shall alternatively use either of the following constructs:
		///         TripleData<DataType> a, b;
		///         a /= (DataType) b;
		///         a.sync();
		/// or
		///         TripleData<DataType> a, b;
		///         a = (DataType) a / (DataType) b;
		/// which recover both a and b.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType>& operator/=(const HardenedData::TripleData<DataType>& val) {
			d1 /= val.d1;
			d2 /= val.d2;
			d3 /= val.d3;
			return *this;  // Return a reference to myself
		}


		/// <summary>
		/// Overwrites the algebraic operator a | b when a is redundant TripleData<DataType> while b is DataType.
		/// It OR-s each replica of the left-hand variable a with the right-hand variable (or constant) b and returns the result as a redundant TripleData<DataType>. Neither a nor b are affected.
		/// When using this operator, the left-hand variable is not verified to be correct; each of the three replicas of the left-hand value is OR-ed as it is with the right-hand variable, for a faster execution. As a consequence, OR-ing a corrupted left-hand value with a right-hand variable, leaves the value corrupted.
		/// To OR and correct a corrupted left-hand value with a right-hand variable (or constant), one shall use the following construct:
		///         TripleData<DataType> a, b, c;
		///         DataType b;
		///         c = (DataType) a | b; // or, alternatively
		///         c = (DataType) a | 0x20;
		/// where type casting TripleData<DataType> a to (DataType) recovers a correct value from its three replicas (by voting). The correct value is then copied identically to the three replicas of c.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType> operator|(const DataType& val) const {
			TripleData<DataType> result;
			result.d1 = d1 | val;
			result.d2 = d2 | val;
			result.d3 = d3 | val;
			return result;
		}


		/// <summary>
		/// Overwrites the logic operator a | b when both a and b are redundant TripleData<DataType>.
		/// It OR-s each replica of the left-hand variable a with the corresponding replica of the right-hand variable b and returns the result as a redundant TripleData<DataType>. Neither a nor b are affected.
		/// When using this operator, neither the left-hand nor the right-hand variables are verified to be correct; each of the three replicas of the left-hand value is OR-ed as it is with the corresponding replica of the right-hand variable, for a faster execution. As a consequence, OR-ing a corrupted left-hand value with a corrupted right-hand variable, leaves the value corrupted.
		/// Note that corruption propagation with OR-s is more risky than with assignation operator = for the following reason: imagine that a has the first replica corrupted, while the second and third replica are correct, while b only has its second replica corrupted. Both a and b can be individually recovered by voting, but OR-ing the first, second and third replicas of a and b together, will return a corrupted first replica (because of a's corruption) together with a corrupted second replica (because of b's corruption), so the result will NOT be recoverable!
		/// To AND and correct a corrupted left-hand value to a corrupted right-hand variable, one shall use the following construct:
		///         TripleData<DataType> a, b, c;
		///         c = (DataType) a | (DataType) b;
		/// where type casting TripleData<DataType> a and b to (DataType) recovers a correct value from the three replicas of both variables (by voting). The correct value is then copied identically to the three replicas of c.
		/// Alternatively, for a faster execution, the following construct:
		///         TripleData<DataType> a, b, c;
		///         c = (DataType) a | b;
		/// will only recover a but not b; the result might be potentially corrupted but recoverable, as at most one replica of the result will be corrupted.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType> operator|(const HardenedData::TripleData<DataType>& val) const {
			TripleData<DataType> result;
			result.d1 = d1 | val.d1;
			result.d2 = d2 | val.d2;
			result.d3 = d3 | val.d3;
			return result;
		}


		/// <summary>
		/// Overwrites the logic operator a |= b when a is redundant TripleData<DataType> while b is DataType.
		/// It OR-s each replica of the left-hand variable a with the right-hand variable (or constant) b, stores the result back to a and returns the result as a redundant TripleData<DataType>. Variable a is affected, while b is not affected.
		/// When using this operator, the left-hand variables is not verified to be correct; each of the three replicas of the left-hand value is OR-ed as it is with the right-hand variable, for a faster execution. As a consequence, OR-ing a corrupted left-hand value with a right-hand variable, leaves the value corrupted.
		/// To OR and correct a corrupted left-hand value to a right-hand variable, one shall use either of the following constructs:
		///         TripleData<DataType> a;
		///         DataType b;
		///         a |= b;
		///         a.sync();
		/// or
		///         TripleData<DataType> a;
		///         DataType b;
		///         a = (DataType) a | b;
		/// which recovers a before using its value.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType>& operator|=(const DataType& val) {
			d1 |= val;
			d2 |= val;
			d3 |= val;
			return *this;  // Return a reference to myself
		}


		/// <summary>
		/// Overwrites the logic operator a |= b when both a and b are redundant TripleData<DataType>.
		/// It OR-s each replica of the left-hand variable a with the corresponding replica of the right-hand variable b, stores the result back to a and returns the result as a redundant TripleData<DataType>. Variable a is affected, while b is not affected.
		/// When using this operator, neither the left-hand nor the right-hand variables are verified to be correct; each of the three replicas of the left-hand value is OR-ed as it is with the corresponding replica of the right-hand variable, for a faster execution. As a consequence, OR-ing a corrupted left-hand value with a corrupted right-hand variable, leaves the value corrupted.
		/// Note that corruption propagation with OR-s is more risky than with assignation operator = for the following reason: imagine that a has the first replica corrupted, while the second and third replica are correct, while b only has its second replica corrupted. Both a and b can be individually recovered by voting, but OR-ing the first, second and third replicas of a and b together, will return a corrupted first replica (because of a's corruption) together with a corrupted second replica (because of b's corruption), so the result will NOT be recoverable!
		/// To OR and correct a corrupted left-hand value with a corrupted right-hand variable, one shall use the following construct:
		///         TripleData<DataType> a, b;
		///         a |= (DataType) b;
		/// where type casting TripleData<DataType> b to (DataType) recovers a correct value from the three replicas of b (by voting). The correct value is then copied identically to the three replicas of c.
		/// If b is corrupted, its correct value is recovered, while if a is corrupted, it will remain corrupted. To have a fully correct value stored into a, one shall alternatively use either of the following constructs:
		///         TripleData<DataType> a, b;
		///         a |= (DataType) b;
		///         a.sync();
		/// or
		///         TripleData<DataType> a, b;
		///         a = (DataType) a | (DataType) b;
		/// which recover both a and b.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType>& operator|=(const HardenedData::TripleData<DataType>& val) {
			d1 |= val.d1;
			d2 |= val.d2;
			d3 |= val.d3;
			return *this;  // Return a reference to myself
		}


		/// <summary>
		/// Overwrites the logic operator a & b when both a and b are redundant TripleData<DataType>.
		/// It AND-s each replica of the left-hand variable a with the corresponding replica of the right-hand variable b and returns the result as a redundant TripleData<DataType>. Neither a nor b are affected.
		/// When using this operator, neither the left-hand nor the right-hand variables are verified to be correct; each of the three replicas of the left-hand value is AND-ed as it is with the corresponding replica of the right-hand variable, for a faster execution. As a consequence, AND-ing a corrupted left-hand value with a corrupted right-hand variable, leaves the value corrupted.
		/// Note that corruption propagation with AND-s is more risky than with assignation operator = for the following reason: imagine that a has the first replica corrupted, while the second and third replica are correct, while b only has its second replica corrupted. Both a and b can be individually recovered by voting, but AND-ing the first, second and third replicas of a and b together, will return a corrupted first replica (because of a's corruption) together with a corrupted second replica (because of b's corruption), so the result will NOT be recoverable!
		/// To AND and correct a corrupted left-hand value to a corrupted right-hand variable, one shall use the following construct:
		///         TripleData<DataType> a, b, c;
		///         c = (DataType) a & (DataType) b;
		/// where type casting TripleData<DataType> a and b to (DataType) recovers a correct value from the three replicas of both variables (by voting). The correct value is then copied identically to the three replicas of c.
		/// Alternatively, for a faster execution, the following construct:
		///         TripleData<DataType> a, b, c;
		///         c = (DataType) a & b;
		/// will only recover a but not b; the result might be potentially corrupted but recoverable, as at most one replica of the result will be corrupted.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType> operator&(const HardenedData::TripleData<DataType>& val) const {
			TripleData<DataType> result;
			result.d1 = d1 & val.d1;
			result.d2 = d2 & val.d2;
			result.d3 = d3 & val.d3;
			return result;
		}


		/// <summary>
		/// Overwrites the algebraic operator a & b when a is redundant TripleData<DataType> while b is DataType.
		/// It AND-s each replica of the left-hand variable a with the right-hand variable (or constant) b and returns the result as a redundant TripleData<DataType>. Neither a nor b are affected.
		/// When using this operator, the left-hand variable is not verified to be correct; each of the three replicas of the left-hand value is AND-ed as it is with the right-hand variable, for a faster execution. As a consequence, AND-ing a corrupted left-hand value with a right-hand variable, leaves the value corrupted.
		/// To AND and correct a corrupted left-hand value with a right-hand variable (or constant), one shall use the following construct:
		///         TripleData<DataType> a, b, c;
		///         DataType b;
		///         c = (DataType) a & b; // or, alternatively
		///         c = (DataType) a & 0x20;
		/// where type casting TripleData<DataType> a to (DataType) recovers a correct value from its three replicas (by voting). The correct value is then copied identically to the three replicas of c.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType> operator&(const DataType& val) const {
			TripleData<DataType> result;
			result.d1 = d1 & val;
			result.d2 = d2 & val;
			result.d3 = d3 & val;
			return result;
		}


		/// <summary>
		/// Overwrites the logic operator a &= b when a is redundant TripleData<DataType> while b is DataType.
		/// It AND-s each replica of the left-hand variable a with the right-hand variable (or constant) b, stores the result back to a and returns the result as a redundant TripleData<DataType>. Variable a is affected, while b is not affected.
		/// When using this operator, the left-hand variables is not verified to be correct; each of the three replicas of the left-hand value is AND-ed as it is with the right-hand variable, for a faster execution. As a consequence, AND-ing a corrupted left-hand value with a right-hand variable, leaves the value corrupted.
		/// To AND and correct a corrupted left-hand value to a right-hand variable, one shall use either of the following constructs:
		///         TripleData<DataType> a;
		///         DataType b;
		///         a &= b;
		///         a.sync();
		/// or
		///         TripleData<DataType> a;
		///         DataType b;
		///         a = (DataType) a & b;
		/// which recovers a before using its value.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType>& operator&=(const DataType& val) {
			d1 &= val;
			d2 &= val;
			d3 &= val;
			return *this;  // Return a reference to myself
		}


		/// <summary>
		/// Overwrites the logic operator a &= b when both a and b are redundant TripleData<DataType>.
		/// It AND-s each replica of the left-hand variable a with the corresponding replica of the right-hand variable b, stores the result back to a and returns the result as a redundant TripleData<DataType>. Variable a is affected, while b is not affected.
		/// When using this operator, neither the left-hand nor the right-hand variables are verified to be correct; each of the three replicas of the left-hand value is AND-ed as it is with the corresponding replica of the right-hand variable, for a faster execution. As a consequence, AND-ing a corrupted left-hand value with a corrupted right-hand variable, leaves the value corrupted.
		/// Note that corruption propagation with AND-s is more risky than with assignation operator = for the following reason: imagine that a has the first replica corrupted, while the second and third replica are correct, while b only has its second replica corrupted. Both a and b can be individually recovered by voting, but AND-ing the first, second and third replicas of a and b together, will return a corrupted first replica (because of a's corruption) together with a corrupted second replica (because of b's corruption), so the result will NOT be recoverable!
		/// To AND and correct a corrupted left-hand value with a corrupted right-hand variable, one shall use the following construct:
		///         TripleData<DataType> a, b;
		///         a &= (DataType) b;
		/// where type casting TripleData<DataType> b to (DataType) recovers a correct value from the three replicas of b (by voting). The correct value is then copied identically to the three replicas of c.
		/// If b is corrupted, its correct value is recovered, while if a is corrupted, it will remain corrupted. To have a fully correct value stored into a, one shall alternatively use either of the following constructs:
		///         TripleData<DataType> a, b;
		///         a &= (DataType) b;
		///         a.sync();
		/// or
		///         TripleData<DataType> a, b;
		///         a = (DataType) a & (DataType) b;
		/// which recover both a and b.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType>& operator&=(const HardenedData::TripleData<DataType>& val) {
			d1 &= val.d1;
			d2 &= val.d2;
			d3 &= val.d3;
			return *this;  // Return a reference to myself
		}


		/// <summary>
		/// Overwrites the algebraic operator a ^ b when a is redundant TripleData<DataType> while b is DataType.
		/// It EXOR-s each replica of the left-hand variable a with the right-hand variable (or constant) b and returns the result as a redundant TripleData<DataType>. Neither a nor b are affected.
		/// When using this operator, the left-hand variable is not verified to be correct; each of the three replicas of the left-hand value is EXOR-ed as it is with the right-hand variable, for a faster execution. As a consequence, EXOR-ing a corrupted left-hand value with a right-hand variable, leaves the value corrupted.
		/// To EXOR and correct a corrupted left-hand value with a right-hand variable (or constant), one shall use the following construct:
		///         TripleData<DataType> a, b, c;
		///         DataType b;
		///         c = (DataType) a ^ b; // or, alternatively
		///         c = (DataType) a ^ 0x20;
		/// where type casting TripleData<DataType> a to (DataType) recovers a correct value from its three replicas (by voting). The correct value is then copied identically to the three replicas of c.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType> operator^(const DataType& val) const {
			TripleData<DataType> result;
			result.d1 = d1 ^ val;
			result.d2 = d2 ^ val;
			result.d3 = d3 ^ val;
			return result;
		}


		/// <summary>
		/// Overwrites the logic operator a ^ b when both a and b are redundant TripleData<DataType>.
		/// It EXOR-s each replica of the left-hand variable a with the corresponding replica of the right-hand variable b and returns the result as a redundant TripleData<DataType>. Neither a nor b are affected.
		/// When using this operator, neither the left-hand nor the right-hand variables are verified to be correct; each of the three replicas of the left-hand value is EXOR-ed as it is with the corresponding replica of the right-hand variable, for a faster execution. As a consequence, EXOR-ing a corrupted left-hand value with a corrupted right-hand variable, leaves the value corrupted.
		/// Note that corruption propagation with EXOR-s is more risky than with assignation operator = for the following reason: imagine that a has the first replica corrupted, while the second and third replica are correct, while b only has its second replica corrupted. Both a and b can be individually recovered by voting, but EXOR-ing the first, second and third replicas of a and b together, will return a corrupted first replica (because of a's corruption) together with a corrupted second replica (because of b's corruption), so the result will NOT be recoverable!
		/// To EXOR and correct a corrupted left-hand value to a corrupted right-hand variable, one shall use the following construct:
		///         TripleData<DataType> a, b, c;
		///         c = (DataType) a ^ (DataType) b;
		/// where type casting TripleData<DataType> a and b to (DataType) recovers a correct value from the three replicas of both variables (by voting). The correct value is then copied identically to the three replicas of c.
		/// Alternatively, for a faster execution, the following construct:
		///         TripleData<DataType> a, b, c;
		///         c = (DataType) a ^ b;
		/// will only recover a but not b; the result might be potentially corrupted but recoverable, as at most one replica of the result will be corrupted.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType> operator^(const HardenedData::TripleData<DataType>& val) const {
			TripleData<DataType> result;
			result.d1 = d1 ^ val.d1;
			result.d2 = d2 ^ val.d2;
			result.d3 = d3 ^ val.d3;
			return result;
		}


		/// <summary>
		/// Overwrites the logic operator a ^= b when a is redundant TripleData<DataType> while b is DataType.
		/// It EXOR-s each replica of the left-hand variable a with the right-hand variable (or constant) b, stores the result back to a and returns the result as a redundant TripleData<DataType>. Variable a is affected, while b is not affected.
		/// When using this operator, the left-hand variables is not verified to be correct; each of the three replicas of the left-hand value is EXOR-ed as it is with the right-hand variable, for a faster execution. As a consequence, EXOR-ing a corrupted left-hand value with a right-hand variable, leaves the value corrupted.
		/// To EXOR and correct a corrupted left-hand value to a right-hand variable, one shall use either of the following constructs:
		///         TripleData<DataType> a;
		///         DataType b;
		///         a ^= b;
		///         a.sync();
		/// or
		///         TripleData<DataType> a;
		///         DataType b;
		///         a = (DataType) a ^ b;
		/// which recovers a before using its value.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType>& operator^=(const DataType& val) {
			d1 ^= val;
			d2 ^= val;
			d3 ^= val;
			return *this;  // Return a reference to myself
		}


		/// <summary>
		/// Overwrites the logic operator a ^= b when both a and b are redundant TripleData<DataType>.
		/// It EXOR-s each replica of the left-hand variable a with the corresponding replica of the right-hand variable b, stores the result back to a and returns the result as a redundant TripleData<DataType>. Variable a is affected, while b is not affected.
		/// When using this operator, neither the left-hand nor the right-hand variables are verified to be correct; each of the three replicas of the left-hand value is EXOR-ed as it is with the corresponding replica of the right-hand variable, for a faster execution. As a consequence, EXOR-ing a corrupted left-hand value with a corrupted right-hand variable, leaves the value corrupted.
		/// Note that corruption propagation with EXOR-s is more risky than with assignation operator = for the following reason: imagine that a has the first replica corrupted, while the second and third replica are correct, while b only has its second replica corrupted. Both a and b can be individually recovered by voting, but EXOR-ing the first, second and third replicas of a and b together, will return a corrupted first replica (because of a's corruption) together with a corrupted second replica (because of b's corruption), so the result will NOT be recoverable!
		/// To EXOR and correct a corrupted left-hand value with a corrupted right-hand variable, one shall use the following construct:
		///         TripleData<DataType> a, b;
		///         a ^= (DataType) b;
		/// where type casting TripleData<DataType> b to (DataType) recovers a correct value from the three replicas of b (by voting). The correct value is then copied identically to the three replicas of c.
		/// If b is corrupted, its correct value is recovered, while if a is corrupted, it will remain corrupted. To have a fully correct value stored into a, one shall alternatively use either of the following constructs:
		///         TripleData<DataType> a, b;
		///         a ^= (DataType) b;
		///         a.sync();
		/// or
		///         TripleData<DataType> a, b;
		///         a = (DataType) a ^ (DataType) b;
		/// which recover both a and b.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType>& operator^=(const HardenedData::TripleData<DataType>& val) {
			d1 ^= val.d1;
			d2 ^= val.d2;
			d3 ^= val.d3;
			return *this;  // Return a reference to myself
		}


		/// <summary>
		/// Overwrites the logic prefix operator ~a when a is redundant TripleData<DataType>.
		/// It complements each replica of the right-hand variable and returns the result as a redundant TripleData<DataType>. Variable a is not affected.
		/// When using this operator, the right-hand variables is not verified to be correct; each of the three replicas of the right-hand value is complemented as it is, for a faster execution. As a consequence, complementing a corrupted right-hand value, returns a corrupted value.
		/// To correct, then complement a corrupted right-hand value, one shall use the following construct:
		///         TripleData<DataType> a;
		///         ~(a.sync());
		/// which recovers a before complementing its value.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// Instead, when the complemented value must casted to a (DataType):
		///         TripleData<DataType> a;
		///         DataType b;
		///         b = ~a;
		/// a faster way could be to first cast, then to complement:
		///         TripleData<DataType> a;
		///         DataType b;
		///         b = ~((DataType) a);
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType> operator~() const {
			TripleData<DataType> result;
			result.d1 = ~ d1;
			result.d2 = ~ d2;
			result.d3 = ~ d3;
			return result;
		}


		/// <summary>
		/// Overwrites the logic operator a >> b when a is redundant TripleData<DataType>, while b is a byte (often, a constant).
		/// It right-shifts each replica of the left-hand variable a by the number of positions defined by the right-hand variable (or constant) b and returns the result as a redundant TripleData<DataType>. Neither a nor b are affected.
		/// When using this operator, the left-hand variable is not verified to be correct; each of the three replicas of the left-hand value is right-shifted as it is by the value of the right-hand variable, for a faster execution. As a consequence, right-shifting a corrupted left-hand value, leaves the value corrupted.
		/// To right-shift and correct a corrupted left-hand value, one shall use the following construct:
		///         TripleData<DataType> a, b, c;
		///         byte b;
		///         c = (DataType) a >> b; // or, alternatively
		///         c = (DataType) a >> 2;
		/// where type casting TripleData<DataType> a to (DataType) recovers a correct value from its three replicas (by voting). The correct value is then copied identically to the three replicas of c.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType> operator>>(const int& val) const {
			TripleData<DataType> result;
			result.d1 = d1 >> val;
			result.d2 = d2 >> val;
			result.d3 = d3 >> val;
			return result;
		}


		/// <summary>
		/// Overwrites the logic operator a << b when a is redundant TripleData<DataType>, while b is a byte (often, a constant).
		/// It left-shifts each replica of the left-hand variable a by the number of positions defined by the right-hand variable (or constant) b and returns the result as a redundant TripleData<DataType>. Neither a nor b are affected.
		/// When using this operator, the left-hand variable is not verified to be correct; each of the three replicas of the left-hand value is left-shifted as it is by the value of the right-hand variable, for a faster execution. As a consequence, left-shifting a corrupted left-hand value, leaves the value corrupted.
		/// To left-shift and correct a corrupted left-hand value, one shall use the following construct:
		///         TripleData<DataType> a, b, c;
		///         byte b;
		///         c = (DataType) a << b; // or, alternatively
		///         c = (DataType) a << 2;
		/// where type casting TripleData<DataType> a to (DataType) recovers a correct value from its three replicas (by voting). The correct value is then copied identically to the three replicas of c.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType> operator<<(const int& val) const {
			TripleData<DataType> result;
			result.d1 = d1 << val;
			result.d2 = d2 << val;
			result.d3 = d3 << val;
			return result;
		}


		/// <summary>
		/// Overwrites the logic operator a >>= b when a is redundant TripleData<DataType> while b is byte.
		/// It right-shifts each replica of the left-hand variable a by the number of positions identified by the right-hand variable (or constant) b, stores the result back to a and returns the result as a redundant TripleData<DataType>. Variable a is affected, while b is not affected.
		/// When using this operator, the left-hand variables is not verified to be correct; each of the three replicas of the left-hand value is right-shifted as it is by the value of the right-hand variable, for a faster execution. As a consequence, left-shifting a corrupted left-hand value, leaves the value corrupted.
		/// To right-shift and correct a corrupted left-hand value, one shall use either of the following constructs:
		///         TripleData<DataType> a;
		///         byte b;
		///         a >>= b;
		///         a.sync();
		/// or
		///         TripleData<DataType> a;
		///         byte b;
		///         a = (DataType) a >> b;
		/// which recovers a before using its value.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType>& operator>>=(const int& val) {
			d1 >>= val;
			d2 >>= val;
			d3 >>= val;
			return *this;  // Return a reference to myself
		}


		/// <summary>
		/// Overwrites the logic operator a >>= b when a is redundant TripleData<DataType> while b is byte.
		/// It left-shifts each replica of the left-hand variable a by the number of positions identified by the right-hand variable (or constant) b, stores the result back to a and returns the result as a redundant TripleData<DataType>. Variable a is affected, while b is not affected.
		/// When using this operator, the left-hand variables is not verified to be correct; each of the three replicas of the left-hand value is left-shifted as it is by the value of the right-hand variable, for a faster execution. As a consequence, left-shifting a corrupted left-hand value, leaves the value corrupted.
		/// To left-shift and correct a corrupted left-hand value, one shall use either of the following constructs:
		///         TripleData<DataType> a;
		///         byte b;
		///         a <<= b;
		///         a.sync();
		/// or
		///         TripleData<DataType> a;
		///         byte b;
		///         a = (DataType) a << b;
		/// which recovers a before using its value.
		/// Because of the longer execution time of voting, it is strongly suggested to use it only when really required.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::TripleData<DataType>& operator<<=(const int& val) {
			d1 <<= val;
			d2 <<= val;
			d3 <<= val;
			return *this;  // Return a reference to myself
		}


		/// <summary>
		/// Overwrites the logic operator a == b when a is redundant TripleData<DataType> while b is DataType.
		/// It first votes a and compares the result against b and returns result as a bool.
		/// Neither variable a nor b is affected.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool operator==(const DataType& val) const {
			return ((DataType) *this == val);
		}


		/// <summary>
		/// Overwrites the logic operator a == b when both a and b are redundant TripleData<DataType>.
		/// It first votes a and b separately, then it compares voting result and returns comparison result as a bool.
		/// Neither variable a nor b is affected.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool operator==(const HardenedData::TripleData<DataType>& val) const {
			return ((DataType) *this == (DataType) val);
		}


		/// <summary>
		/// Overwrites the logic operator a != b when a is redundant TripleData<DataType> while b is DataType.
		/// It first votes a and compares the result against b and returns result as a bool.
		/// Neither variable a nor b is affected.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool operator!=(const DataType& val) const {
			return ((DataType) *this != val);
		}


		/// <summary>
		/// Overwrites the logic operator a == b when both a and b are redundant TripleData<DataType>.
		/// It first votes a and b separately, then it compares voting result and returns comparison result as a bool.
		/// Neither variable a nor b is affected.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool operator!=(const HardenedData::TripleData<DataType>& val) const {
			return ((DataType) *this != (DataType) val);
		}


		/// <summary>
		/// Attempts to recover the correct value of the data stored into the class, starting from its three replicas.
		/// If the three replicas have not been corrupted, or at most one of them has been corrupted, the sync() : DataType operation will recover the correct value by majority voting, then restores this value into the corrupted replica (the other replicas are not touched).
		/// If two or more replicas are corrupted, the error is unrecoverable and the sync() : DataType will return the value of the first replica.
		/// This operation differs from only voting (as performed by the casting operator operator operator DataType()) as the latter only returns a potentially correct value without correcting the three replicas, while the former also recovers the data into the class.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		DataType sync() {
			if (d1 == d2) {
			  if (d1 == d3) {
			    return d1;
			    }
			  else {
			    d3 = d1;
			    return d1;
			    }
			} else {
			  if (d1 == d3) {
			    d2 = d1;
			    return d1;
			    }
			  else if (d2 == d3) {
			    d1 = d3;
			    return d1;
			    }
			  else
			    return d1;
			  }
		}


		/// <summary>
		/// Returns true when a correct value can be recovered, by using either the type casting operator operator operator DataType() or the sync() : DataType operator.
		/// In other words, it returns true when either the three replicas are identical or at most one replica is different from the other two.
		/// When all the three replicas differ, it returns false.
		/// There is a potential situation (usually with very low probability) that two errors have affected two different replicas in the same way; in this case, the check() : bool operator will return true, but the correct value CANNOT be recovered.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool check() const {
			if (d1 == d2) return true;
			if (d1 == d3) return true;
			if (d2 == d3) return true;
			else return false;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		DataType vote() const {
			return (DataType) (*this);
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool voteBit(DataType mask) const {
			if ((d1&mask)==(d2&mask)) return (d1&mask);
			else if ((d1&mask)==(d3&mask)) return (d1&mask);
			else return (d2&mask);
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool agree() const {
			return check();
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool agreeBit(DataType mask) const {
			if (((d1&mask) == (d2&mask)) && ((d1&mask) == (d3&mask)))
			  return true;
			else
			  return false;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::WHO disagree() const {
			if (agree()) return NONE;
			if (d1==d2) return CC;
			if (d1==d3) return BB;
			return AA;
		}


 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		HardenedData::WHO disagreeBit(DataType mask) const {
			if (agreeBit(mask)) return NONE;
			if ((d1&mask)==(d2&mask)) return CC;
			if ((d1&mask)==(d3&mask)) return BB;
			return AA;
		}


		/// <summary>
		/// Returns first replica of the data.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
	DataType& a() {
			return d1;
		}


		/// <summary>
		/// Returns second replica of the data.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		DataType& b() {
			return d2;
		}


		/// <summary>
		/// Returns third replica of the data.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		DataType& c() {
			return d3;
		}


		/// <summary>
		/// Returns either:
		/// first replica of the data, if who==AA;
		/// second replica of the data, if who==BB;
		/// third replica of the data, if who==CC;
		/// first replica of the data, for any other value of who; 
		///  
		///  
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		DataType& x(const HardenedData::WHO who) {
			switch (who){
			case AA: return d1; break;
			case BB: return d2; break;
			case CC: return d3; break;
			default: return d1; break;
			}
		}


		/// <summary>
		/// Sets to 1, in the three replicas of data stored in the class, all the bits which are set to 1 in the argument mask, while the other bits are not affected.
		/// Exactly equivalent to the operator operator operator|=(val : TripleData  &) : TripleData  &. In practice, the two forms:
		///         TripleData<DataType> a, b;
		///         a.set(b);
		///         a |= b;
		/// have identical effect. Despite identical, both are left for compatibility with old programs.
		/// For more details and side effects of this operation, see documentation of operator operator|=(val : TripleData  &) : TripleData  &.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		void set(const HardenedData::TripleData<DataType>& mask) {
			(*this) |= mask;
		}


		/// <summary>
		/// Sets to 1, in the three replicas of data stored in the class, all the bits which are set to 1 in the argument mask, while the other bits are not affected.
		/// Exactly equivalent to the operator operator operator|=(val : DataType &) : TripleData  &. In practice, the two forms:
		///         TripleData<DataType> a;
		///         DataType b;
		///         a.set(b);
		///         a |= b;
		/// have identical effect. Despite identical, both are left for compatibility with old programs.
		/// For more details and side effects of this operation, see documentation of operator operator|=(val : DataType &) : TripleData  &.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		void set(const DataType& mask) {
			(*this) |= mask;
		}


		/// <summary>
		/// Reets to 0, in the three replicas of data stored in the class, all the bits which are set to 1 in the argument mask, while the other bits are not affected.
		/// Exactly equivalent to the operator operator operator&=(val : TripleData  &) : TripleData  & associated with operator operator~() : TripleData  &. In practice, the two forms:
		///         TripleData<DataType> a, b;
		///         a.reset(b);
		///         a &= ~b;
		/// have identical effect. Despite identical, both are left for compatibility with old programs.
		/// For more details and side effects of this operation, see documentation of operator operator&=(val : TripleData  &) : TripleData  & and operator operator~() : TripleData  &.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		void reset(const HardenedData::TripleData<DataType>& mask) {
			(*this) &= ~mask;
		}


		/// <summary>
		/// Sets to 0, in the three replicas of data stored in the class, all the bits which are set to 1 in the argument mask, while the other bits are not affected.
		/// Exactly equivalent to the operator operator operator&=(val : DataType &) : TripleData  & associated with ~. In practice, the two forms:
		///         TripleData<DataType> a;
		///         DataType b;
		///         a.reset(b);
		///         a &= ~b;
		/// have identical effect. Despite identical, both are left for compatibility with old programs.
		/// For more details and side effects of this operation, see documentation of operator operator&=(val : DataType &) : TripleData  &.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		void reset(const DataType& mask) {
			(*this) &= ~mask;
		}


		/// <summary>
		/// Writes the content of argument val into the three replicas of data stored in the class.
		/// Exactly equivalent to the operator operator operator=(val : TripleData  &) : TripleData  &. In practice, the two forms:
		///         TripleData<DataType> a, b;
		///         a.write(b);
		///         a = b;
		/// have identical effect. Despite identical, both are left for compatibility with old programs.
		/// For more details and side effects of this operation, see documentation of operator operator=(val : TripleData  &) : TripleData  &.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		void write(const HardenedData::TripleData<DataType>& val) {
			(*this) =val;
		}


		/// <summary>
		/// Writes the content of argument val into the three replicas of data stored in the class.
		/// Exactly equivalent to the operator operator operator=(val : DataType) : TripleData  &. In practice, the two forms:
		///         TripleData<DataType> a;
		///         DataType b;
		///         a.write(b);
		///         a = b;
		/// have identical effect. Despite identical, both are left for compatibility with old programs.
		/// For more details and side effects of this operation, see documentation of operator operator=(val : DataType) : TripleData  &.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		void write(const DataType& val) {
			(*this) =val;
		}


		/// <summary>
		/// Writes partially, in the three replicas of data stored in the class, the value of the argument data, but only in all the bits which are set to 1 in the argument mask, while the other bits are not affected.
		/// In practice:
		///         TripleData<DataType> a, data, mask;
		///         a = 0x1357;
		///         data = 0xFF00;
		///         mask = 0x0FF0;
		///         a.write(data, mask);
		/// leaves first nibble (1) and last nibble (7) unchanged, then writes F0 in the second and third nibble.
		/// So, at the end, a will contain the value 0x1F07.
		/// This operation is a combination of the operator operator=(val : TripleData  &) : TripleData  &,  operator operator&(val : TripleData  &) : TripleData  &,,  operator operator|(val : TripleData  &) : TripleData  &  and operator operator~() : TripleData  &. For more details and side effects of this operation, see documentation of these operators..
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		void write(HardenedData::TripleData<DataType>& data, HardenedData::TripleData<DataType>& mask) {
			(*this) = (data & mask) | ((*this) & ~mask);
		}


		/// <summary>
		/// Writes partially, in the three replicas of data stored in the class, the value of the argument data, but only in all the bits which are set to 1 in the argument mask, while the other bits are not affected.
		/// In practice:
		///         TripleData<DataType> a,;
		///         DataType data, mask;
		///         a = 0x1357;
		///         data = 0xFF00;
		///         mask = 0x0FF0;
		///         a.write(data, mask);
		/// leaves first nibble (1) and last nibble (7) unchanged, then writes F0 in the second and third nibble.
		/// So, at the end, a will contain the value 0x1F07.
		/// This operation is a combination of the operator operator=(val : DataType) : TripleData  &,  operator operator&(val : DataType &) : TripleData  &, operator operator|(val : DataType &) : TripleData  & and ~. For more details and side effects of these operations, see documentation of these operators.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		void write(DataType& data, DataType& mask) {
			(*this) = (data & mask) | ((*this) & (DataType)(~mask));
		}


		/// <summary>
		/// Returns true if and only if the bits which are set in data are set also in the data stored in the class.
		/// In practice:
		///         TripleData<DataType> a, data;
		///         a = 0x1F07;
		///         data = 0x1F00;
		///         a.test(data);
		/// will return true, as all the bits which are set in data are also set in a, while if:
		///         a = 0x15D7;
		///         data = 0xFF00;
		///         a.test(data);
		/// will return false, as some of the bits which are set in data are NOT set in a.
		/// This operation is equivalent to test(data, data).
		/// For more details and side effects of this operation, see documentation of these operators.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool test(const HardenedData::TripleData<DataType>& data) const {
			return (*this).test(data, data);
		}


		/// <summary>
		/// Returns true if and only if the bits which are set in data are set also in the data stored in the class.
		/// In practice:
		///         TripleData<DataType> a, data;
		///         a = 0x1F07;
		///         data = 0x1F00;
		///         a.test(data);
		/// will return true, as all the bits which are set in data are also set in a, while if:
		///         a = 0x15D7;
		///         data = 0xFF00;
		///         a.test(data);
		/// will return false, as some of the bits which are set in data are NOT set in a.
		/// This operation is equivalent to test(data, data).
		/// For more details and side effects of this operation, see documentation of these operators.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool test(const DataType& data) const {
			return (*this).test(data, data);
		}


		/// <summary>
		/// Returns true if and only if ALL bits of data stored in the class, corresponding to bits set to 1 in argument mask are identical to data.
		/// In practice:
		///         TripleData<DataType> a, data, mask;
		///         a = 0x1F07;
		///         data = 0xFF00;
		///         mask = 0x0FF0;
		///         a.test(data, mask);
		/// will return true, as all the bits in the second and third nibble (namely, all those bits where mask is 1) of a are identical to data, while if:
		///         a = 0x15D7;
		///         data = 0xFF00;
		///         mask = 0x0FF0;
		///         a.test(data, mask);
		/// will return false, as some of the bits in the second and third nibble (namely, all those bits where mask is 1) of a are NOT identical to data.
		/// This operation is a combination of the operator operator=(val : TripleData  &) : TripleData  &,  operator operator&(val : TripleData  &) : TripleData  &,,  operator operator|(val : TripleData  &) : TripleData  &  and operator operator~() : TripleData  &. For more details and side effects of this operation, see documentation of these operators.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool test(const HardenedData::TripleData<DataType>& data, const HardenedData::TripleData<DataType>& mask) const {
			return (((*this) & mask) == (data & mask));
		}


		/// <summary>
		/// Returns true if and only if ALL bits of data stored in the class, corresponding to bits set to 1 in argument mask are identical to data.
		/// In practice:
		///         TripleData<DataType> a;
		///         DataType data, mask;
		///         a = 0x1F07;
		///         data = 0xFF00;
		///         mask = 0x0FF0;
		///         a.test(data, mask);
		/// will return true, as all the bits in the second and third nibble (namely, all those bits where mask is 1) of a are identical to data, while if:
		///         a = 0x15D7;
		///         data = 0xFF00;
		///         mask = 0x0FF0;
		///         a.test(data, mask);
		/// will return false, as some of the bits in the second and third nibble (namely, all those bits where mask is 1) of a are NOT identical to data.
		/// This operation is a combination of the operator operator=(val : DataType) : TripleData  &,  operator operator&(val : DataType &) : TripleData  &, operator operator|(val : DataType &) : TripleData  & and ~. For more details and side effects of these operations, see documentation of these operators.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool test(const DataType& data, const DataType& mask) const {
			return (((*this) & mask) == (DataType)(data & mask));
		}


		/// <summary>
		/// Returns true if and only if ALL bits of data stored in the class are identical to data.
		/// In practice:
		///         TripleData<DataType> a, data;
		///         a = 0x1F07;
		///         data = 0x1F07;
		///         a.test(data);
		/// will return true, as all the bits of a are identical to data, while if:
		///         a = 0x15D7;
		///         data = 0xFF00;
		///         a.test(data);
		/// will return false, as some of the bits of a are NOT identical to data.
		/// This operation is a combination of the operator operator=(val : TripleData  &) : TripleData  &,  operator operator&(val : TripleData  &) : TripleData  &,,  operator operator|(val : TripleData  &) : TripleData  &  and operator operator~() : TripleData  &. For more details and side effects of this operation, see documentation of these operators.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool testExact(const HardenedData::TripleData<DataType>& data) const {
			return ((*this) == data);
		}


		/// <summary>
		/// Returns true if and only if ALL bits of data stored in the class are identical to data.
		/// In practice:
		///         TripleData<DataType> a;
		///         a = 0x1F07;
		///         data = 0x1F07;
		///         a.test(data);
		/// will return true, as all the bits of a are identical to data, while if:
		///         a = 0x15D7;
		///         data = 0xFF00;
		///         a.test(data);
		/// will return false, as some of the bits of a are NOT identical to data.
		/// This operation is a combination of the operator operator=(val : DataType) : TripleData  &,  operator operator&(val : DataType &) : TripleData  &, operator operator|(val : DataType &) : TripleData  & and ~. For more details and side effects of these operations, see documentation of these operators.
		/// </summary>
 
		public:  
#pragma inline = forced
__attribute__((always_inline))
		bool testExact(const DataType& data) const {
			return ((*this) == data);
		}


 
		public: void swap(HardenedData::TripleData<DataType>& data) {
			DataType tmp;
			tmp = d1;
			d1 = data.d1;
			data.d1 = tmp;
			tmp = d2;
			d2 = data.d2;
			data.d2 = tmp;
			tmp = d3;
			d3 = data.d3;
			data.d3 = tmp;
		}

	};
}
#endif
